<?php

namespace Drupal\tmgmt_cdt;

use Drupal\Component\Serialization\Json;

/**
 * Helper class.
 */
class TmgmtCdtHelper {

  const SIMULATION = 'SIMULATION';
  const PRODUCTION = 'PRODUCTION';

  const NAME = 'Translation Centre (CdT)';
  const DEFAULT_REQUEST_IDENTIFIER = '0000/unknown';
  const TOKEN_TYPE_DRUPAL_TOKEN_MEDIA = 'drupal_token_media';
  const TOKEN_TYPE_DRUPAL_TOKEN_SIMPLE = 'drupal_token_simple';
  const TOKEN_TYPE_HTML_A = 'html_a';
  const TOKEN_TYPE_HTML_IMG = 'html_img';
  const TOKEN_TYPE_HTML_SCRIPT = 'html_script';

  const API_STATUS_COMPLETED = 'COMPLETED';
  const API_STATUS_QUOTED_PENDING_APPROVAL = 'QUOTED - PENDING APPROVAL';
  const API_STATUS_CANCELLED = 'CANCELLED';

  const API_COMMENT_URLS = [
    'DO_NOT_ADAPT_EXTERNAL_URLS' => 'Do not adapt external urls',
    'ADAPT_EXTERNAL_URLS' => 'Adapt external urls',
  ];

  /**
   * Check if is a json.
   */
  public static function isJson($data) {
        // @codingStandardsIgnoreStart
        try {
            // must be \json_decode function.
            if (\is_string($data) && \is_array(\json_decode($data, true)) && (json_last_error() === JSON_ERROR_NONE)) {
                return true;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            return false;
        }
        // @codingStandardsIgnoreEnd
  }

  /**
   * Get default options.
   */
  public static function defaultOptions() {

    $options = [

      'tmgmt_cdt.api_mode' => self::SIMULATION,
      'tmgmt_cdt.ready' => FALSE,
      'tmgmt_cdt.debug' => FALSE,

      'tmgmt_cdt.business_reference_data' => new \stdClass(),
      'tmgmt_cdt.api_url' => '',
      'tmgmt_cdt.api_username' => '',
      'tmgmt_cdt.api_password' => '',

      'tmgmt_cdt.curl_proxy_host' => '',
      'tmgmt_cdt.curl_proxy_port' => '',
      'tmgmt_cdt.curl_proxy_user' => '',
      'tmgmt_cdt.curl_proxy_pass' => '',
      'tmgmt_cdt.curl_proxy_user_agent' => '',
      'tmgmt_cdt.curl_options' => '{"CURLOPT_TIMEOUT":15,"CURLOPT_RETURNTRANSFER":true,"CURLOPT_SSL_VERIFYHOST":0,"CURLOPT_SSL_VERIFYPEER":0}',

      'tmgmt_cdt.max_pages_per_request' => 10,
      'tmgmt_cdt.use_token_html_a' => FALSE,
      'tmgmt_cdt.use_token_html_img' => FALSE,
      'tmgmt_cdt.use_token_html_script' => TRUE,
      'tmgmt_cdt.use_token_drupal' => TRUE,
      'tmgmt_cdt.html_check' => TRUE,
      'tmgmt_cdt.show_quotation' => FALSE,
      'tmgmt_cdt.xsd_link' => 'http://static.cdt.europa.eu/webtranslations/schemas/Drupal-Translation-V8-0.xsd',
      'tmgmt_cdt.disable_xsd_validation' => FALSE,
      'tmgmt_cdt.xsd_source' => '',

      'tmgmt_cdt.api_access_token_expire' => 0,
      'tmgmt_cdt.api_access_token' => '',

    ];

    return $options;
  }

  /**
   * File get content using translator proxy settings.
   */
  public static function curlGetFileContents($url) {
    try {

      $proxy_host = \Drupal::state()->get('tmgmt_cdt.curl_proxy_host');
      $proxy_port = (int) \Drupal::state()->get('tmgmt_cdt.curl_proxy_port');
      $proxy_user = \Drupal::state()->get('tmgmt_cdt.curl_proxy_user');
      $proxy_pass = \Drupal::state()->get('tmgmt_cdt.curl_proxy_pass');
      $proxy_user_agent = \Drupal::state()->get('tmgmt_cdt.curl_proxy_user_agent');
      $curl_options = Json::decode(\Drupal::state()->get('tmgmt_cdt.curl_options'));

      $curl = curl_init();
      $curlopt[CURLOPT_RETURNTRANSFER] = 1;
      $curlopt[CURLOPT_URL] = $url;

      if (!empty($proxy_host)) {
        $curlopt[CURLOPT_PROXY] = $proxy_host;
        $curlopt[CURLOPT_PROXYPORT] = $proxy_port;
      }

      if (!empty($proxy_user)) {
        $curlopt[CURLOPT_PROXYUSERPWD] = $proxy_user . ':' . $proxy_pass;
      }

      if (!empty($proxy_user_agent)) {
        $curlopt[CURLOPT_USERAGENT] = $proxy_user_agent;
      }

      foreach ($curl_options as $curl_option => $curl_value) {
        $curlopt[constant($curl_option)] = $curl_value;
      }

      curl_setopt_array($curl, $curlopt);
      $raw_data = curl_exec($curl);
      if (curl_errno($curl)) {
        throw new \Exception(curl_error($curl));
      }

      curl_close($curl);
      unset($curl);

      if ($raw_data) {
        return $raw_data;
      }
      else {
        return FALSE;
      }

    }
    catch (\Exception $e) {
      // \Drupal::messenger()->addError($e->getMessage());
      return FALSE;
    }

  }

  /**
   * Show if the module status is ready or not.
   */
  public static function showStatus($showReady = FALSE) {
    if ($showReady) {
      if (!(bool) \Drupal::state()->get('tmgmt_cdt.ready')) {
        \Drupal::messenger()->addWarning(t('<strong>@name</strong> translator is <strong>not ready</strong>.', ['@name' => self::NAME]));
      }
      else {
        \Drupal::messenger()->addStatus(t('<strong>@name</strong> translator is <strong>ready</strong>.', ['@name' => self::NAME]));
      }
    }

    if (\Drupal::state()->get('tmgmt_cdt.api_mode') === self::SIMULATION && (bool) \Drupal::state()->get('tmgmt_cdt.ready')) {
      \Drupal::messenger()->addWarning(
            t(
                '<strong>@name</strong> use the mode <strong>@mode</strong>.',
                [
                  '@name' => self::NAME,
                  '@mode' => \Drupal::state()->get('tmgmt_cdt.api_mode'),
                ]
            )
        );
    }

  }

  /**
   * Extract year and number.
   */
  public static function splitRequestIdentifier(string $request_number) {
    $request_ids = explode('/', trim($request_number));
    $request_year = @trim($request_ids[0]);
    $request_number = @trim($request_ids[1]);
    return [
      'requestyear' => $request_year,
      'requestnumber' => $request_number,
    ];
  }

  /**
   * Delete array key by path.
   */
  public static function arrayDeleteByPath(array &$data, string $path = '', string $separator = '][') {

    $segments = explode($separator, $path);
    $size = count($segments);
    $cur =& $data;
    $count = 0;
    foreach ($segments as $segment) {
      $count++;
      if (!isset($cur[$segment])) {
        $cur[$segment] = [];
      }
      if ($count === $size) {
        unset($cur[$segment]);
        return NULL;
      }
      $cur =& $cur[$segment];
    }
  }

  /**
   * Check job rules.
   */
  public static function checkJob($job) {
    $errors = [];
    try {
      $volume = (int) $job->getCounts()['pages_language'];
      $max = (int) \Drupal::state()->get('tmgmt_cdt.max_pages_per_request');
      if ($volume > $max) {
        $errors[] = t(
              'The translation volume is too large (@volume pages). The maximum per request is @max pages.',
              ['@volume' => $volume, '@max' => $max]
          );
      }

      return $errors;
    }
    catch (\Exception $e) {
      $errors[] = $e->getMessage();
      return $errors;
    }
  }

  /**
   * Check job items.
   */
  public static function checkCartItems() {
    $errors = [];
    $characters = 0;
    $volume = 0;
    $max = (int) \Drupal::state()->get('tmgmt_cdt.max_pages_per_request');

    foreach (\Drupal::service('tmgmt.cart')->getJobItemsFromCart() as $job_item) {
      if (!$job_item) {
        \Drupal::messenger()->addError('errors on cart validation');
        continue;
      }
      $job_item->onCart = TRUE;

      foreach (\Drupal::service('tmgmt.data')->filterTranslatable($job_item->getSourceData()) as $data) {
        $characters += TmgmtCdtFormatHelper::characterCount(@$data['#text']);
      }
      $job_item->onCart = FALSE;
    }
    $volume = ceil($characters / 750) * 0.5;
    if ($volume > $max) {
      $errors[] = t(
            'The translation volume is too large (@volume pages). The maximum per request is @max pages.',
            ['@volume' => $volume, '@max' => $max]
        );
    }
    return $errors;
  }

  /**
   * Return job by request_identifier.
   */
  public static function jobByRequestIdentifier(string $request_identifier) {
    if ($request_identifier === self::DEFAULT_REQUEST_IDENTIFIER) {
      return FALSE;
    }

    $query = \Drupal::entityQuery('tmgmt_job')
      ->condition('translator', 'tmgmt_cdt')
      ->condition('cdt_request_identifier', $request_identifier);
    $result = $query->execute();
    if (empty($result)) {
      return FALSE;
    }

    $ids = array_keys($result);
    $job = \Drupal::entityTypeManager()->getStorage('tmgmt_job')->load((int) reset($ids));

    return $job;
  }

}
